﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;
using FlowerShopDataModels.Models;
using Microsoft.EntityFrameworkCore;

namespace FlowerShopDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return new();
            }

            using var context = new FlowerShopDatabase();

            return context.Shops.Include(x => x.Bouquets)
                                .ThenInclude(x => x.Bouquet)
                                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.Name == model.Name) ||
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }

            using var context = new FlowerShopDatabase();

            return context.Shops.Include(x => x.Bouquets).ThenInclude(x => x.Bouquet).Where(x => x.Name.Contains(model.Name)).ToList().Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();

            return context.Shops.Include(x => x.Bouquets).ThenInclude(x => x.Bouquet).ToList().Select(x => x.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            using var transaction = context.Database.BeginTransaction();

            try
            {
                var newShop = Shop.Create(context, model);
                if (newShop == null)
                {
                    return null;
                }

                if (context.Shops.Any(x => x.Name == newShop.Name))
                {
                    throw new Exception("Название магазина уже занято");
                }

                context.Shops.Add(newShop);
                context.SaveChanges();
                transaction.Commit();

                return newShop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            using var transaction = context.Database.BeginTransaction();

            try
            {
                var shop = context.Shops.Include(x => x.Bouquets).FirstOrDefault(x => x.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }

                shop.Update(model);
                context.SaveChanges();

                if (model.ShopBouquets.Count > 0)
                {
                    shop.UpdateBouquets(context, model);
                }

                transaction.Commit();

                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            var shop = context.Shops.Include(x => x.Bouquets).FirstOrDefault(x => x.Id == model.Id);

            if (shop != null)
            {
                context.Shops.Remove(shop);
                context.SaveChanges();

                return shop.GetViewModel;
            }

            return null;
        }

        public bool SellBouquet(IBouquetModel model, int count)
        {
            using var context = new FlowerShopDatabase();
            using var transaction = context.Database.BeginTransaction();

            foreach (var shopBouquets in context.ShopBouquets.Where(x => x.BouquetId == model.Id))
            {
                var min = Math.Min(count, shopBouquets.Count);
                shopBouquets.Count -= min;
                count -= min;

                if (count <= 0)
                {
                    break;
                }
            }

            if (count == 0)
            {
                context.SaveChanges();
                transaction.Commit();
            }

            else
            {
                transaction.Rollback();
            }

            if (count > 0)
            {
                return false;
            }

            return true;
        }
    }
}
